// #region Copyright
// Copyright ArtfulBits Inc. 2005 - 2010. All rights reserved.
// 
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// info@artfulbits.com. Re-distribution in any form is strictly
// prohibited. Any infringement will be prosecuted under applicable laws.
// #endregion
package aiCharts.systemGraph;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.os.Handler;

import com.artfulbits.aiCharts.Base.ChartSeries;

/**
 * Extract CPU usage runtime value.
 * 
 * @author Oleksandr Kucherenko
 */
public class CpuRuntimeValues extends ValuesCollector
{
	/* Constants */
	/** Pattern for extracting CPU usage statistics. */
	private final static String PARSE_REGEX = "cpu[\\t ]+([0-9]+)[\\t ]+([0-9]+)[\\t ]+([0-9]+)[\\t ]+([0-9]+)(.*)";
	/** Virtual file system entry that contains CPU usage statistics. */
	private final static String PROC_STAT = "/proc/stat";

	/* Members */
	/** USER time */
	private long mUtimeOld = 0;
	/** NICE time */
	private long mNtimeOld = 0;
	/** SYSTEM time */
	private long mStimeOld = 0;
	/** IDLE time */
	private long mItimeOld = 0;

	/**
	 * @param handler
	 * @param series
	 */
	public CpuRuntimeValues( Handler handler, ChartSeries series )
	{
		super( handler, series );
	}

	/*
	 * (non-Javadoc)
	 * @see com.artfulbits.system.graph.ValuesCollector#getRuntimeValue()
	 */
	@Override
	protected double getRuntimeValue()
	{
		double cpuUsage = 0d;

		try
		{
			BufferedReader reader = new BufferedReader( new FileReader( PROC_STAT ), 1024 );
			final String statistics = reader.readLine();
			reader.close();

			Pattern c = Pattern.compile( PARSE_REGEX );
			Matcher m = c.matcher( statistics );

			if( m.matches() )
			{
				/* USER, NICE, SYSTEM, IDLE */
				long uTime = Long.parseLong( m.group( 1 ) );
				long nTime = Long.parseLong( m.group( 2 ) );
				long sTime = Long.parseLong( m.group( 3 ) );
				long iTime = Long.parseLong( m.group( 4 ) );

				/* Calculate CPU usage values */
				long totalDelta = Math.abs( ( uTime + nTime + sTime + iTime ) - 
						( mUtimeOld + mNtimeOld + mStimeOld + mItimeOld ) );
				long valueTime = Math.abs( ( uTime + nTime + sTime ) - 
						( mUtimeOld + mNtimeOld + mStimeOld ) );

				// save old values for next call
				mUtimeOld = uTime; /* USER */
				mNtimeOld = nTime; /* NICE */
				mStimeOld = sTime; /* SYSTEM */
				mItimeOld = iTime; /* IDLE */

				cpuUsage = percent( 0, totalDelta, valueTime );
			}
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}

		return cpuUsage;
	}

	/**
	 * Calculate percent proportion.
	 * 
	 * @param min
	 *          Minimal value that represents 0 (zero) percents.
	 * @param max
	 *          Maximum value that represents 100 percents.
	 * @param value
	 *          Current value which between MIN and MAX.
	 * @return Percent representation of current value.
	 */
	private static final double percent( long min, long max, long value )
	{
		final double dMax = max - min;
		final double dValue = value - min;

		return ( double )( dValue * 100 / dMax );
	}
}
