// #region Copyright
// Copyright ArtfulBits Inc. 2005 - 2010. All rights reserved.
// 
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// info@artfulbits.com. Re-distribution in any form is strictly
// prohibited. Any infringement will be prosecuted under applicable laws.
// #endregion
package aiCharts.systemGraph;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.AlertDialog.Builder;
import android.os.Bundle;
import android.os.Debug;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

import com.artfulbits.aiCharts.ChartView;
import com.artfulbits.aiCharts.Base.ChartArea;
import com.artfulbits.aiCharts.Base.ChartSeries;
import com.artfulbits.aiCharts.Types.ChartTypes;
import com.artfulbits.aiCharts.Utilities.AxisFormattingHelper;

public class MainActivity extends Activity
{
	/* Constants */
	private final static int ABOUT_ITEM_ID = 1;
	
	/** CPU Area name. */
	private final static String CPU = "CPU";
	
	/** MEMORY area name. */
	private final static String MEMORY = "Memory";
	
	/** BATTERY area name. */
	private final static String BATTERY = "Battery";
	
	/**
	 * Index of Start profile menu item. Warning: do not use index equal to
	 * zero!!!
	 */
	private final static int MENU_START = 10;
	
	/**
	 * Index of Start profile for 30 seconds menu item. Warning: do not use index
	 * equal to zero!!!
	 */
	private final static int MENU_START_30SEC = 11;
	
	/**
	 * Index of Stop profiling menu item. Warning: do not use index equal to
	 * zero!!!
	 */
	private final static int MENU_STOP = 12;
	
	/* Members */
	/** Handlers that used for executing runtime values collectors. */
	private final Handler[] mHandlers = new Handler[]{new Handler(),
			new Handler(),new Handler()};
	
	/** References on runtime values collectors. */
	private final ValuesCollector[] mCollectors = new ValuesCollector[ 3 ];
	
	/** Reference on chart view instance. */
	private ChartView mChartView;
	
	/** True - we in profiling state, otherwise False. */
	private boolean mIsProfiling = false;;
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		
		// load main.xml layout
		setContentView(R.layout.main);
		
		// use chart ID for getting instance reference
		mChartView = (ChartView)findViewById(R.id.chartView);
		
		// create chart data series
		ChartSeries seriesCPU = new ChartSeries(ChartTypes.Area);
		ChartSeries seriesMemory = new ChartSeries(ChartTypes.Area);
		ChartSeries seriesBattery = new ChartSeries(ChartTypes.Area);
		
		// attach series to chart view
		mChartView.getSeries().add(seriesCPU);
		mChartView.getSeries().add(seriesMemory);
		mChartView.getSeries().add(seriesBattery);
		
		// create chart areas for displaying series
		ChartArea areaCPU = new ChartArea(CPU);
		ChartArea areaMemory = new ChartArea(MEMORY);
		ChartArea areaBattery = new ChartArea(BATTERY);
		
		// apply percents formatting for axis numbers
		AxisFormattingHelper.percentsAxis(areaCPU.getDefaultYAxis());
		AxisFormattingHelper.percentsAxis(areaMemory.getDefaultYAxis());
		AxisFormattingHelper.percentsAxis(areaBattery.getDefaultYAxis());
		
		areaBattery.getDefaultYAxis().setTitle(
				getString(R.string.battery_axis_title));
		areaMemory.getDefaultYAxis()
				.setTitle(getString(R.string.memory_axis_title));
		areaCPU.getDefaultYAxis().setTitle(getString(R.string.cpu_axis_title));
		
		AxisFormattingHelper.CompactAxis(areaBattery.getDefaultYAxis());
		AxisFormattingHelper.CompactAxis(areaMemory.getDefaultYAxis());
		AxisFormattingHelper.CompactAxis(areaCPU.getDefaultYAxis());
		
		// attach areas to view
		mChartView.getAreas().add(areaCPU);
		mChartView.getAreas().add(areaMemory);
		mChartView.getAreas().add(areaBattery);
		
		// associate series with area - binding by chart area name
		seriesCPU.setArea(CPU);
		seriesMemory.setArea(MEMORY);
		seriesBattery.setArea(BATTERY);
		
		// customize area view
		seriesCPU.setLineWidth(2);
		seriesCPU.setBorderColor(0xFFFFFFFF);
		seriesMemory.setLineWidth(2);
		seriesMemory.setBorderColor(0xFFFFFFFF);
		seriesBattery.setLineWidth(2);
		seriesBattery.setBorderColor(0xFFFFFFFF);
		
		// create instances of runtime data collectors
		mCollectors[0] = new CpuRuntimeValues(mHandlers[0], seriesCPU);
		mCollectors[1] = new MemoryRuntimeValues(mHandlers[1], seriesMemory);
		mCollectors[2] = new BatteryRuntimeValues(this, mHandlers[2], seriesBattery);
		
		// start runtime data collecting
		for(int i = 0; i < mHandlers.length; i++)
		{
			mCollectors[i].reset();
			mHandlers[i].postDelayed(mCollectors[i], 0);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onPause()
	 */
	@Override
	protected void onPause()
	{
		super.onPause();
		
		// shutdown all collectors
		for(int i = 0; i < mCollectors.length; i++)
		{
			mCollectors[i].shutdown();
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onResume()
	 */
	@Override
	protected void onResume()
	{
		super.onResume();
		
		// start runtime data collecting and reset values collecting
		for(int i = 0; i < mHandlers.length; i++)
		{
			mCollectors[i].reset();
			mHandlers[i].postDelayed(mCollectors[i], 0);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onMenuItemSelected(int, android.view.MenuItem)
	 */
	@Override
	public boolean onMenuItemSelected(int featureId, MenuItem item)
	{
		if(item != null)
		{
			switch(item.getItemId())
			{
				case MENU_START_30SEC:
					Runnable autoStop = new Runnable()
					{
						@Override
						public void run()
						{
							if(mIsProfiling)
							{
								Debug.stopMethodTracing();
								mIsProfiling = false;
								Log.v("STAT", "Profiling stopped after 30 seconds...");
							}
						}
					};
					/* force stop in 30 seconds */
					new Handler().postDelayed(autoStop, 30 * 1000);
					// WARNING: do not place BREAK or RETURNs here.
					
				case MENU_START: /* Start profiling */
					Log.v("STAT", "Profiling started...");
					// NOTE: output file is "/sdcard/aicharts.trace"
					
					/* tests SD card access */
					if(!isSdPresent())
					{
						Toast.makeText(this, R.string.no_sdcard_message, Toast.LENGTH_LONG)
								.show();
						
						break;
					}
					
					/* start profiling */
					Debug.startMethodTracing("aicharts");
					mIsProfiling = true;
					return true;
					
				case MENU_STOP: /* Stop profiling */
					Debug.stopMethodTracing();
					Log.v("STAT", "Profiling stopped...");
					mIsProfiling = false;
					return true;
					
				case ABOUT_ITEM_ID:
					showDialog(ABOUT_ITEM_ID);
					return true;
			}
		}
		
		return super.onMenuItemSelected(featureId, item);
	}
	
	/**
	 * Check is SD Card available on device.
	 * 
	 * @return True is SD card found, otherwise False.
	 */
	public static boolean isSdPresent()
	{
		return Environment.getExternalStorageState().equals(
				Environment.MEDIA_MOUNTED);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Activity#onPrepareOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onPrepareOptionsMenu(Menu menu)
	{
		menu.clear();
		
		if(mIsProfiling) // add STOP
		{
			menu.add(0, MENU_STOP, 0, R.string.menu_stop_profiler);
			menu.add(0, ABOUT_ITEM_ID, 0, getString(R.string.about_menu_item_title));
		}
		else
		// add START
		{
			menu.add(0, MENU_START, 0, R.string.menu_start_profiler);
			menu.add(0, MENU_START_30SEC, 0, R.string.menu_start_30sec);
			menu.add(0, ABOUT_ITEM_ID, 0, getString(R.string.about_menu_item_title));
		}
		
		return super.onPrepareOptionsMenu(menu);
	}
	
	@Override
	protected Dialog onCreateDialog(int id)
	{
		Dialog result = null;
		Builder builder;
		
		switch(id)
		{
			case ABOUT_ITEM_ID:
				builder = new AlertDialog.Builder(this);
				
				builder.setTitle(R.string.about_menu_item_title);
				builder.setMessage(R.string.dlg_about_msg);
				
				builder.setPositiveButton(R.string.dlg_ok_btn_title, null);
				
				result = builder.create();
				break;
			
			default:
				result = super.onCreateDialog(id);
				break;
		}
		
		return result;
	}
	
}
