// #region Copyright
// Copyright ArtfulBits Inc. 2005 - 2010. All rights reserved.
// 
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// info@artfulbits.com. Re-distribution in any form is strictly
// prohibited. Any infringement will be prosecuted under applicable laws.
// #endregion
package aiCharts.systemGraph;

import android.os.Handler;

import com.artfulbits.aiCharts.Base.ChartPoint;
import com.artfulbits.aiCharts.Base.ChartSeries;

/**
 * Base class for all Runtime values collectors.
 * 
 * @author Oleksandr Kucherenko
 */
public abstract class ValuesCollector implements Runnable
{
	/* Constants */
	/** Delay in millisecond between next value adding into series. */
	protected final static int DELAY = 500;
	/** Quantity of dots that we should keep in series. */
	protected final static int MAX_VALUES = 60;

	/* Members */
	/** Reference on data series instance used for plotting. */
	protected final ChartSeries mSeries;
	/** Reference on handler that keep reference on thread instance. */
	protected final Handler mHandler;
	/** True - thread in shutdown state, otherwise False. */
	protected boolean mShutdown = false;
	/** X-coordinate of next point in series. */
	protected double mX;

	/* Constructor */
	/**
	 * Create thread and specify series instance which used for data displaying.
	 * 
	 * @param handler
	 *          Reference on handler that keep reference on thread. Used for
	 *          delayed updates.
	 * @param series
	 *          Reference on series instance used for CPU graph plotting.
	 */
	public ValuesCollector( final Handler handler, final ChartSeries series )
	{
		mHandler = handler;
		mSeries = series;
	}

	/* Methods */
	/** Restart points collecting from zero. */
	public void reset()
	{
		mX = 0;
		mSeries.getPoints().clear();
		mShutdown = false;
	}

	/** Shutdown values collecting . */
	public void shutdown()
	{
		mShutdown = true;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run()
	{
		double value = getRuntimeValue();

		// X coordinate is replaced by counter value
		ChartPoint point = new ChartPoint( mX++, value );

		// add new value into series points collection
		mSeries.getPoints().add( point );

		// trim series size if we rich it limit
		if( mSeries.getPoints().size() > MAX_VALUES )
			mSeries.getPoints().removeAt( 0 );

		// process shutdown properly
		if( !mShutdown )
			mHandler.postDelayed( this, DELAY );
	}

	/**
	 * Extract runtime value on call.
	 * 
	 * @return value calculated in runtime and valid at the moment of call.
	 */
	protected abstract double getRuntimeValue();
}
