// #region Copyright
// Copyright ArtfulBits Inc. 2005 - 2010. All rights reserved.
// 
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// info@artfulbits.com. Re-distribution in any form is strictly
// prohibited. Any infringement will be prosecuted under applicable laws.
// #endregion
package aiCharts.Demo.Utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import android.content.Context;
import android.content.SharedPreferences;
import android.database.sqlite.SQLiteDatabase;

import com.artfulbits.aiCharts.ChartView;
import com.artfulbits.aiCharts.Base.*;
import com.artfulbits.aiCharts.Base.ChartLayoutElement.*;
import com.artfulbits.aiCharts.Types.ChartTypes;

public class ChartPreferencesHandler
{
	// #region Members
private static SQLiteDatabase m_database;
	
	private int m_ID;
	
	public String[] columnNames;
	
	public String[] seriesTypes;
	
	public Dock legendDock = Dock.Right;
	
	public Alignment legendAlignment = Alignment.Near;
	
	public boolean legendVisibile;
	
	public int[] palette;
	
	public static final String CHART_PROPERTIES = "chart properties";
	
	public static final String VIEW_ID = "viewId";
	
	private static final String c_databaseName = "data.db";
	
	private static final String c_databasePath = "/data/data/aiCharts.Demo/databases/";
	
	private static final String c_tableName = "RandomData";
	
	private static final String COLUMN_NAME_ALIAS = "column";
	
	private static final String SERIES_TYPE_ALIAS = "type";
	
	private static final String LEGEND_DOCK_ALIAS = "dock";
	
	private static final String LEGEND_ALIGNMENT_ALIAS = "alignment";
	
	private static final String LEGEND_VISIBILITY_ALIAS = "visibility";
	
	private static final String PALETTE_ALIAS = "color";
	
	private static final String CHART_VIEW_PREF_ALIAS = "cv";
	
	private static final ArrayList<String> c_stringBufferArray = new ArrayList<String>();
	
	private static final ArrayList<Integer> c_integerBufferArray = new ArrayList<Integer>();
	// #endregion
	
	// #region Public methods
	/**
	 * Initializes a new instance of chart preferences handler that will load
	 * previously stored preferences for specified view.
	 * 
	 * @param viewID zero-based view ID.
	 */
	public ChartPreferencesHandler(int viewID)
	{
		if(viewID < 0)
		{
			throw new IllegalArgumentException(
					"Chart preferences cannot be initialized with" + viewID);
		}
		m_ID = viewID;
	}
	
	/**
	 * Fills view with properties.
	 * 
	 * @param chartView target view.
	 */
	public void transferToView(ChartView chartView)
	{
		ChartNamedCollection<ChartSeries> seriesCollection = chartView.getSeries();
		
		// Avoid data reassignment if series is already filled with correct points.
		if(columnNames != null && seriesCollection.size() != columnNames.length)
		{
			seriesCollection.clear();
			ChartSeries series;
			for(int i = 0; i < columnNames.length; i++)
			{
				series = new ChartSeries("Series" + (i + 1), ChartTypes
						.get(seriesTypes[i]));
				seriesCollection.add(series);
				series.getPoints().setData(getDatabase(chartView.getContext()),
						c_tableName, new String[]{columnNames[i]});
			}
			getDatabase(chartView.getContext()).close();
		}
		else if(seriesTypes != null)
		{
			// Making sure series types are correct.
			for(int i = 0; i < seriesTypes.length; i++)
			{
				seriesCollection.get(i).setType(ChartTypes.get(seriesTypes[i]));
			}
		}
		chartView.getLegends().get(0).setAlignment(legendAlignment);
		chartView.getLegends().get(0).setDock(legendDock);
		chartView.getLegends().get(0).setVisible(legendVisibile);
		chartView.getChart().setPalette(new ChartPalette(palette));
	}
	
	/**
	 * Gets chart view's properties and saves them.
	 * 
	 * @param view target view.
	 */
	public void transferFromView(ChartView chartView)
	{
		ChartNamedCollection<ChartSeries> seriesCollection = chartView.getSeries();
		
		ChartSeries series;
		for(int i = 0; i < seriesCollection.size(); i++)
		{
			series = seriesCollection.get(i);
			seriesTypes[i] = series.getType().getName();
		}
		legendAlignment = chartView.getLegends().get(0).getAlignment();
		legendDock = chartView.getLegends().get(0).getDock();
		legendVisibile = chartView.getLegends().get(0).isVisible();
		palette = chartView.getChart().getPalette().getValues();
	}
	
	/**
	 * Loads chart's properties from preferences.
	 * 
	 * @param context handle to shares preferences.
	 */
	public boolean loadSettingsFromPreferences(Context context)
	{
		// Retrieving shared preferences mapped to current chart view by ID.
		SharedPreferences settings = context.getSharedPreferences(
				CHART_VIEW_PREF_ALIAS + m_ID, 0);
		
		if(!settings.contains(SERIES_TYPE_ALIAS + "0")
				|| !settings.contains(COLUMN_NAME_ALIAS + "0"))
		{
			return false;
		}
		legendAlignment = Alignment.values()[settings.getInt(
				LEGEND_ALIGNMENT_ALIAS, 0)];
		legendDock = Dock.values()[settings.getInt(LEGEND_DOCK_ALIAS, 0)];
		legendVisibile = settings.getBoolean(LEGEND_VISIBILITY_ALIAS, false);
		
		// Retrieving mapped SQL database column names and series types.
		short k = 0;
		String buff;
		// Read all existing column names.
		while((buff = settings.getString(COLUMN_NAME_ALIAS + (k++), null)) != null)
		{
			c_stringBufferArray.add(buff);
		}
		// Save column names locally.
		if(c_stringBufferArray.size() > 0)
		{
			columnNames = new String[ c_stringBufferArray.size() ];
			columnNames = c_stringBufferArray.toArray(columnNames);
		}
		c_stringBufferArray.clear();
		// Read all saved series types.
		k = 0;
		while((buff = settings.getString(SERIES_TYPE_ALIAS + (k++), null)) != null)
		{
			c_stringBufferArray.add(buff);
		}
		// Save series types locally.
		if(c_stringBufferArray.size() > 0)
		{
			seriesTypes = new String[ c_stringBufferArray.size() ];
			c_stringBufferArray.toArray(seriesTypes);
		}
		c_stringBufferArray.clear();
		
		// Read all palettes.
		k = 0;
		int color;
		while((color = settings.getInt(PALETTE_ALIAS + k++, -1)) != -1)
		{
			c_integerBufferArray.add(color);
		}
		// Save palettes locally.
		if(c_integerBufferArray.size() > 0)
		{
			palette = new int[ c_integerBufferArray.size() ];
			for(int i = 0; i < palette.length; i++)
			{
				palette[i] = c_integerBufferArray.get(i);
			}
			c_integerBufferArray.clear();
		}
		return true;
	}
	
	public void saveSettingsToPreferences(Context context)
	{
		SharedPreferences.Editor editor = context.getSharedPreferences(
				CHART_VIEW_PREF_ALIAS + m_ID, 0).edit();
		// Clearing previously written preferences.
		editor.clear();
		// Writing legend's settings.
		editor.putInt(LEGEND_ALIGNMENT_ALIAS, legendAlignment.ordinal());
		editor.putInt(LEGEND_DOCK_ALIAS, legendDock.ordinal());
		editor.putBoolean(LEGEND_VISIBILITY_ALIAS, legendVisibile);
		
		// Writing corresponding SQL database columns.
		for(int i = 0; i < columnNames.length; i++)
		{
			editor.putString(COLUMN_NAME_ALIAS + i, columnNames[i]);
		}
		// Writing current series types.
		for(int i = 0; i < seriesTypes.length; i++)
		{
			editor.putString(SERIES_TYPE_ALIAS + i, seriesTypes[i]);
		}
		// Writing applied palette.
		for(int i = 0; i < palette.length; i++)
		{
			editor.putInt(PALETTE_ALIAS + i, palette[i]);
		}
		// Saving changes.
		editor.commit();
	}
	
	private static SQLiteDatabase getDatabase(Context context)
	{
		if(m_database == null || !m_database.isOpen())
		{
			String databaseFilePath = c_databasePath + c_databaseName;
			// Making necessary paths.
			new File(c_databasePath).mkdirs();
			// Resorting to existing database instead of writing a new one.
			if(new File(databaseFilePath).exists())
			{
				m_database = SQLiteDatabase.openDatabase(databaseFilePath, null,
						SQLiteDatabase.OPEN_READONLY);
				return m_database;
			}
			
			try
			{
				InputStream in = context.getAssets().open(c_databaseName);
				OutputStream out;
				out = new FileOutputStream(databaseFilePath);
				
				byte[] buf = new byte[ 1024 ];
				int len;
				while((len = in.read(buf)) > 0)
				{
					out.write(buf, 0, len);
				}
				out.close();
				in.close();
			}
			catch(IOException e)
			{
				e.printStackTrace();
			}
			m_database = SQLiteDatabase.openDatabase(databaseFilePath, null,
					SQLiteDatabase.OPEN_READONLY);
		}
		
		return m_database;
	}
	
	/**
	 * Gets saved view ID.
	 * 
	 * @return zero-based ID.
	 */
	public int getViewID()
	{
		return m_ID;
	}
	
	/**
	 * Determined whether preferences are valid.
	 * 
	 * @return true if preferences for chart view are valid.
	 */
	public boolean valid()
	{
		return seriesTypes != null && palette != null;
	}
	
	/**
	 * Clears all preferences saved.
	 * 
	 * @param context
	 */
	public static void clearPreferences(Context context)
	{
		short k = 0;
		SharedPreferences preferences = context.getSharedPreferences(
				CHART_VIEW_PREF_ALIAS + k++, 0);
		while(preferences.contains(SERIES_TYPE_ALIAS + "0"))
		{
			preferences.edit().clear().commit();
			preferences = context
					.getSharedPreferences(CHART_VIEW_PREF_ALIAS + k++, 0);
		}
	}
	// #endregion
}
